// ProvaDrDaq.cpp : main project file.

#include "stdafx.h"

#include <windows.h>
#include <conio.h>
#include <stdio.h>
#include "usbDrDaqApi.h"

#define SAMPLES_CYCLE 1000 // 1000 samples per cycle (generated signal)
#define DUTY_CYCLES_NUM 15 // number of duty cycles
#define INJ_TIME_MIN_IN 500 // us
#define INJ_TIME_MAX_IN 7500 // us

int32_t				scale_to_mv;
uint16_t			max_adc_value;
int16_t				g_handle;
int16_t				isReady;
int16_t				d1State, d2State, d3State, d4State = 0;
PICO_STATUS			status;
USB_DRDAQ_INPUTS	channel = USB_DRDAQ_CHANNEL_SCOPE;

using namespace System;

void channelScaling()
{
	int16_t nScales, currentScale, i;
	int8_t names[1000];
	int8_t	ch;
	int32_t selectedScaleNo;

	UsbDrDaqGetScalings(g_handle, channel, &nScales, &currentScale, names, 1000);

	printf("%d scale(s) available for channel %d:\n\n", nScales, channel);

	i = 0;

	while (i < 1000)
	{
		if (names[i] == 13) //Carriage return
		{
			printf("\n");
		}
		else
		{
			printf("%c", names[i]);
		}
		i++;
	}

	if (nScales > 1)
	{
		printf("\ncurrent scale: %d\n\n", currentScale);

		printf("Press 'C' to change scale or any other key to continue\n");

		ch = toupper(_getch());

		if (ch == 'C')
		{
			do
			{
				printf("Select scale (0 to %d):", nScales - 1);
				scanf_s("%d", &selectedScaleNo);
				printf("\n");
			} while (selectedScaleNo < 0 || selectedScaleNo > nScales - 1);

			UsbDrDaqSetScalings(g_handle, channel, (int16_t)selectedScaleNo);
		}
	}
}


// Finds start (1->0) and end (0->1) index inside a float array
bool find_start_end_indexes(float* samples, unsigned int samples_numb, unsigned int* start_sample_ptr, unsigned int* end_sample_ptr){
	if (!samples_numb) return false; // no samples
	float signal_run_ave = samples[0];
	float signal_old = signal_run_ave;
	unsigned int status = 0; // 0=before start, 1=found start, 2=found end
	for (unsigned int i = 0; i < samples_numb; i++){
		if ((status == 0) && ((samples[i] - signal_run_ave) < -500)){ // difference higher than 0.5V, going down
			*start_sample_ptr = i;
			status++; // 1
		}
		if ((status == 1) && ((samples[i] - signal_run_ave) > 500)){ // going up
			*end_sample_ptr = i;
			status++; // 2
		}
		signal_run_ave = (float)0.3 * samples[i] + (float)0.7 * signal_old; // running average
		signal_old = samples[i]; // updates old
		if (status == 2) return true; // found both start and end
	}
	return false; // not found both start and end
}


#define N_TRIGGER_MEAS 10
unsigned int collect_block_triggered()
{
	uint32_t	nSamples = 10000;
	int16_t		nChannels = 1;
	uint32_t	nSamplesPerChannel = 10000;
	uint32_t	nSamplesCollected;
	float		samples[10000];
	uint32_t	usForBlock = 20000;
	uint16_t	overflow;
	uint32_t	triggerIndex = 0;
	int32_t		threshold = 1000;

	status = UsbDrDaqSetTrigger(g_handle, 1, 0, 0, channel, 1, (int16_t)500, (uint16_t)10, (float)-10.0);
	status = UsbDrDaqSetInterval(g_handle, &usForBlock, nSamples, &channel, nChannels);
	
	unsigned int start_sample;
	unsigned int end_sample;
	unsigned int inj_us_meas[N_TRIGGER_MEAS];
	unsigned long inj_us_meas_ave = 0; // average value

	for (unsigned int i = 0; i < N_TRIGGER_MEAS; i++){
		
		// Starts the trigger, and waits until unit is ready. Then, acquires values.
		status = UsbDrDaqRun(g_handle, nSamples, BM_SINGLE);
		isReady = 0;
		while (isReady == 0 && (!_kbhit()))
		{
			status = UsbDrDaqReady(g_handle, &isReady);
		}
		nSamplesCollected = nSamplesPerChannel;
		status = UsbDrDaqGetValuesF(g_handle, samples, &nSamplesCollected, &overflow, &triggerIndex);

		// Searches for start and end index
		find_start_end_indexes(samples, nSamplesCollected, &start_sample, &end_sample);
		inj_us_meas[i] = (end_sample - start_sample) * 2; // one sample every 2 us
		inj_us_meas_ave += inj_us_meas[i];

	}
	inj_us_meas_ave /= N_TRIGGER_MEAS;
	//printf("Start=%d. End=%d. Time=%d us. ", start_sample, end_sample, (unsigned int)inj_us_meas_ave);

	status = UsbDrDaqStop(g_handle);
	return inj_us_meas_ave;
}

// Creates a step (1->0) waveform, which is 1 before the duty cycle, and zero after
void create_waveform_samples_array(int16_t* arbitraryWaveform, uint16_t arbitraryWaveformSize, uint8_t duty_cycle_perc){
	// Create waveform with proper duty cycle
	for (unsigned int i = 0; i < arbitraryWaveformSize; i++){
		if (i < (arbitraryWaveformSize * duty_cycle_perc / (uint32_t)100)){ // duty
			arbitraryWaveform[i] = 1000; // max
		}
		else{
			arbitraryWaveform[i] = 0; // min
		}
	}
}

int main(array<System::String ^> ^args)
{

	int8_t	info[20];
	int16_t	requiredSize = 0;

	printf("USB DrDAQ driver example program\n");
	printf("Version 1.2\n\n");

	printf("\nOpening the device...\n");
	status = UsbDrDaqOpenUnit(&g_handle);
	
	if (status != 0)
	{
		printf("Unable to open device\nPress any key\n");
		_getch();
		return 0;
	}
	else
	{

		printf("Device opened successfully:\n\n");
		status = UsbDrDaqGetUnitInfo(g_handle, info, 20, &requiredSize, PICO_VARIANT_INFO);
		printf("Model:\t\t\t %s\n", info);
		status = UsbDrDaqGetUnitInfo(g_handle, info, 20, &requiredSize, PICO_BATCH_AND_SERIAL);
		printf("Serial Number:\t\t %s\n", info);
		status = UsbDrDaqGetUnitInfo(g_handle, info, 20, &requiredSize, PICO_CAL_DATE);
		printf("Calibration Date:\t %s\n", info);
		status = UsbDrDaqGetUnitInfo(g_handle, info, 20, &requiredSize, PICO_USB_VERSION);
		printf("USB Version:\t\t %s\n", info);
		status = UsbDrDaqGetUnitInfo(g_handle, info, 20, &requiredSize, PICO_HARDWARE_VERSION);
		printf("Hardware version:\t %s\n", info);
		status = UsbDrDaqGetUnitInfo(g_handle, info, 20, &requiredSize, PICO_DRIVER_VERSION);
		printf("USBDrDAQ.dll version:\t %s\n", info);
		status = UsbDrDaqGetUnitInfo(g_handle, info, 20, &requiredSize, PICO_KERNEL_VERSION);
		printf("Kernel version:\t\t %s\n", info);
		float min, max;
		short places, divider;
		status = UsbDrDaqGetChannelInfo(g_handle, &min, &max, &places, &divider, channel);
		printf("Min=%f. Max=%f. Places=%d. Divider=%d.\n", min, max, places, divider);
		
		int32_t offsetVoltage = 0; // Offset voltage
		uint32_t pkToPk = 3000000; // Peak to peak voltage
		int16_t arbitraryWaveform[SAMPLES_CYCLE];
		int16_t arbitraryWaveformSize = SAMPLES_CYCLE;
		int32_t updateRate = 1000;
		UsbDrDaqStopSigGen(g_handle);
		
		Sleep(100);

		// File handling
		FILE* fp = NULL;
		fopen_s(&fp, "export.csv", "w");
		fprintf(fp, "Rpm,Duty[%%],Input[us],Output[us],Increment[%%],\n");

		// For each RPM (cycle time)
		unsigned int duty_cycles[DUTY_CYCLES_NUM] = { 1, 2, 5, 10, 15, 20, 25, 30, 35, 40, 45, 50, 60, 70, 80};
		for (unsigned int i = 1; i <= 10; i++){
		
			// For each duty cycle
			for (unsigned int j = 0; j < DUTY_CYCLES_NUM; j++){

				// Creates waveform with defined duty cycle
				create_waveform_samples_array(arbitraryWaveform, SAMPLES_CYCLE, duty_cycles[j]); // 1000 samples as default

				// Writes info on the display
				double RPM = (double)1200 * i; // at i = 1, 1 cycle = 100000us
				unsigned int inj_us_in = 1000 * duty_cycles[j] / i; // injection time input (us)

				// Checks if the injection time is in the proper range (physically acceptable)
				if ((inj_us_in >= INJ_TIME_MIN_IN) && (inj_us_in <= INJ_TIME_MAX_IN)){

					// Print info on the screen
					printf("RPM = %d. ", (unsigned int)RPM);
					printf("Duty cycle = %d %%. ", duty_cycles[j]);
					printf("Inj time in = %d us. ", inj_us_in);

					// Sets signal generation, and waits to stabilize
					status = UsbDrDaqSetSigGenArbitrary(g_handle, offsetVoltage, pkToPk, arbitraryWaveform, arbitraryWaveformSize, (int32_t)SAMPLES_CYCLE * 10 * i);
					Sleep(200); // In order to stabilize

					// Measures injection time
					unsigned int inj_us_meas = collect_block_triggered();
					float perc_increment = (float)((double)100 * ((double)inj_us_meas - (double)inj_us_in) / (double)inj_us_in);
					printf("Inj time out = %d us. ", inj_us_meas);
					printf("Increase = %f %%.\n", perc_increment);

					// Stops signal (prepares to next cycle)
					//Sleep(100);
					UsbDrDaqSetSigGenBuiltIn(g_handle, 0, 0, 0, USB_DRDAQ_DC);
					UsbDrDaqStopSigGen(g_handle); // Set the signal to 0
					//Sleep(100);

					// Export measured data to file
					fprintf(fp, "%f,", RPM);
					fprintf(fp, "%d,", duty_cycles[j]);
					fprintf(fp, "%d,", inj_us_in);
					fprintf(fp, "%d,", inj_us_meas);
					fprintf(fp, "%f,\n", perc_increment);
				}
			}
		}
		fclose(fp);
	}

	// Finished
	printf("\nPress a key to exit...");
	getchar();
	UsbDrDaqStopSigGen(g_handle);
	Sleep(100);
	UsbDrDaqCloseUnit(g_handle);
    return 0;
}
